import torch
import torch.nn.functional as F
from tqdm import tqdm
from torch.optim import Optimizer


@torch.no_grad()
def accuracy_model(model, testloader, device):
    # To disable Dropout and BatchNormalization
    model.eval()
    correct = 0
    total = 0
    # since we're not training, we don't need to calculate the gradients for our outputs
    with torch.no_grad():
        for images, labels in tqdm(testloader):
            # to run on gpu
            images, labels = images.to(device), labels.to(device)
            # calculate outputs by running images through the network
            outputs = model(images)
            # the class with the highest energy is what we choose as prediction
            _, predicted = torch.max(outputs.data, 1)
            total += labels.size(0)
            correct += (predicted == labels).sum().item()
    return 100 * correct / total


@torch.no_grad()
def predictions(loader, model, path, device = "cuda:0"):
    # Load the parameters and network state
    model.load_state_dict(torch.load(path))
    all_probs = None
    with torch.no_grad():  # todo: must be useless
        for inputs, targets in loader:
            # get log probs
            log_probs = F.log_softmax(model(inputs.to(device)), dim=1)
            # get preds
            probs = torch.exp(log_probs)
            if all_probs is None:
                all_probs = probs
            else:
                all_probs = torch.cat((all_probs, probs))
    return all_probs
